import json
from django.views import View
from django.db.models import Q

from apps.ipmi.models import IpmiEvent
from libs import json_response, auth
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger

ITEMS_PER_PAGE = 10


class IpmiEventView(View):

    def get(self, request):
        # 获取查询参数
        EID = request.GET.get('id')
        ip = request.GET.get('ip')
        start_time = request.GET.get('start_time')
        end_time = request.GET.get('end_time')
        page = int(request.GET.get('page', 1))  # 获取当前页数，默认为第1页
        size = int(request.GET.get('size', ITEMS_PER_PAGE))

        # 构建查询条件
        conditions = []
        if ip:
            conditions.append(Q(ip__contains=ip))
        if EID:
            conditions.append(Q(EID__contains=EID))
        if start_time and end_time:
            conditions.append(Q(created_at__gte=start_time))
            conditions.append(Q(created_at__lte=end_time))
        if start_time and not end_time:
            conditions.append(Q(created_at__gte=start_time))
        if not start_time and end_time:
            conditions.append(Q(created_at__lte=end_time))

        # 执行查询，按time倒序，ip正序排序
        event_list = IpmiEvent.objects.filter(*conditions).order_by('-created_at', 'ip')

        # 创建一个 Paginator 对象，提供预先计算好的计数值
        paginator = Paginator(event_list, size) 
        total = paginator.count

        # 获取该页的对象，执行数据库查询
        events = paginator.get_page(page)
        # 序列化查询结果
        events = ([{"id": event.EID, 
                    "ip": event.ip, 
                    "Timestamp": event.created_at, 
                    "SensorType": event.stype, 
                    "EventType": event.etype, 
                    "EventDirection": event.edir, 
                    "EventInterpretation": event.eint, 
                    "Description": event.desc} 
                    for event in events])

        # 计算总页数（假设数据库中总条数已知）
        total_pages = (total + size - 1) // size

        # 返回结果，包括分页信息
        response_data = {
            "events": events,
            "total": total,
            "total_pages": total_pages,
            "current_page": page,
             }

        return json_response(response_data)


    def post(self, request):
        if request.method == 'POST':
            json_data = json.loads(request.body)
            try:
                if len(json_data) > 0:
                    for key,value in json_data.items():
                        print(key,value)
                        IpmiEvent.objects.create(
                            EID = value['id'],
                            ip = value['ip'],
                            created_at=value['Timestamp'],
                            stype=value['SensorType'],
                            etype=value['EventType'],
                            edir=value['EventDirection'],
                            eint=value['EventInterpretation'],
                            desc = value['Description'])
                    return json_response()
            except Exception as e:
                return json_response(error=e)
        else:
            return json_response(error='method not allowed')
